!******************************************************************************!
!                                                                              !
!    lcFSFa.f90                                                                !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcFSFa.f90                                     |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2015                                 |            !
!        |       | UFO version: 0.1                               |            !
!        |       | Generation date: 2020-07-29 13:16             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcFSFa
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29
  contains
    subroutine loopFSFa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,      &
    riMaxOut, rankInc)
      use class_vertices
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(FSFa0)
!       ProjP(3,1)*c0
!       ProjM(3,1)*c1
        pl(:) = PL1(:) + PL2(:)
        x0 = co(1)*w2(0)
        x6 = co(0)*w2(0)
        x12 = co(0)*m
        x13 = cima*w2(0)
        x15 = co(1)*m
        do j = riMaxIn, 0, -1
          x1 = wl(2,j)*x0
          x2 = cima*x1
          x3 = wl(3,j)*x0
          x4 = cima*x3
          x5 = -x4
          x7 = wl(0,j)*x6
          x8 = -cima*x7
          x9 = wl(1,j)*x6
          x10 = cima*x9
          x11 = -x10
          x14 = co(0)*wl(1,j)
          x16 = co(0)*wl(0,j)
          wp(0,j) = x13*(-co(1)*(pl(0)*wl(2,j) + pl(3)*wl(3,j)) + wl(0,j)*x12)
          wp(1,j) = x13*(-co(1)*(pl(1)*wl(3,j) + pl(2)*wl(2,j)) + wl(1,j)*x12)
          wp(2,j) = x13*(-pl(1)*x16 + pl(3)*x14 + wl(2,j)*x15)
          wp(3,j) = x13*(-pl(0)*x14 + pl(2)*x16 + wl(3,j)*x15)
          if(rankInc .lt. 1) cycle
          sp2(0,0) = -x2
          sp2(0,1) = x5
          sp2(0,2) = x8
          sp2(0,3) = x11
          sp2(1,0) = x4
          sp2(1,1) = x2
          sp2(1,2) = x11
          sp2(1,3) = x8
          sp2(2,0) = x3
          sp2(2,1) = -x1
          sp2(2,2) = -x9
          sp2(2,3) = x7
          sp2(3,0) = x2
          sp2(3,1) = x5
          sp2(3,2) = x8
          sp2(3,3) = x10
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      case(FSFa1)
!       ProjM(3,1)*c0
        pl(:) = PL1(:) + PL2(:)
        x0 = co(0)*w2(0)
        x6 = cima*x0
        do j = riMaxIn, 0, -1
          x1 = wl(2,j)*x0
          x2 = cima*x1
          x3 = wl(3,j)*x0
          x4 = cima*x3
          x5 = -x4
          wp(0,j) = -x6*(pl(0)*wl(2,j) + pl(3)*wl(3,j))
          wp(1,j) = -x6*(pl(1)*wl(3,j) + pl(2)*wl(2,j))
          wp(2,j) = m*x2
          wp(3,j) = m*x4
          if(rankInc .lt. 1) cycle
          sp2(0,0) = -x2
          sp2(0,1) = x5
          sp2(0,2) = 0
          sp2(0,3) = 0
          sp2(1,0) = x4
          sp2(1,1) = x2
          sp2(1,2) = 0
          sp2(1,3) = 0
          sp2(2,0) = x3
          sp2(2,1) = -x1
          sp2(2,2) = 0
          sp2(2,3) = 0
          sp2(3,0) = x2
          sp2(3,1) = x5
          sp2(3,2) = 0
          sp2(3,3) = 0
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      case(FSFa2)
!       ProjP(3,1)*c0
        pl(:) = PL1(:) + PL2(:)
        x0 = co(0)*w2(0)
        x7 = cima*x0
        do j = riMaxIn, 0, -1
          x1 = wl(0,j)*x0
          x2 = cima*x1
          x3 = -x2
          x4 = wl(1,j)*x0
          x5 = cima*x4
          x6 = -x5
          wp(0,j) = m*x2
          wp(1,j) = m*x5
          wp(2,j) = -pl(1)*wl(0,j)*x7 + pl(3)*wl(1,j)*x7
          wp(3,j) = x7*(-pl(0)*wl(1,j) + pl(2)*wl(0,j))
          if(rankInc .lt. 1) cycle
          sp2(0,0) = 0
          sp2(0,1) = 0
          sp2(0,2) = x3
          sp2(0,3) = x6
          sp2(1,0) = 0
          sp2(1,1) = 0
          sp2(1,2) = x6
          sp2(1,3) = x3
          sp2(2,0) = 0
          sp2(2,1) = 0
          sp2(2,2) = -x4
          sp2(2,3) = x1
          sp2(3,0) = 0
          sp2(3,1) = 0
          sp2(3,2) = x3
          sp2(3,3) = x5
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      end select 
    end subroutine loopFSFa
end module lcFSFa
